"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.parseAsCustomType = exports.parseAsNull = exports.parseAsNumber = exports.parseAsString = exports.parseAsBoolean = void 0;
var option_1 = require("../option");
var parser_1 = require("./parser");
var util_1 = require("./util");
var parseAsNull = function (t) {
    return (0, parser_1.createPrimitiveParser)({
        t: t,
        expectedType: 'null',
        isMatchingType: function () { return t === null; }
    });
};
exports.parseAsNull = parseAsNull;
var parseAsString = function (t) {
    var isMatchingType = function () { return (0, util_1.isEqual)('string', typeof t); };
    var orElseGetEmptyString = function () { return (!isMatchingType() ? '' : t); };
    return __assign(__assign({}, (0, parser_1.createPrimitiveParser)({
        t: t,
        isMatchingType: isMatchingType,
        expectedType: 'string'
    })), { orElseGetEmptyString: orElseGetEmptyString, exactlyAs: function (s) { return (__assign(__assign({}, (0, option_1.createExact)({
            t: t,
            e: s,
            expectedType: 'string'
        })), { orElseGetEmptyString: orElseGetEmptyString })); } });
};
exports.parseAsString = parseAsString;
var parseAsBoolean = function (t) {
    var expectedType = 'boolean';
    var isMatchingType = function () { return (0, util_1.isEqual)(expectedType, typeof t); };
    var orElseGet = function (b) { return (!isMatchingType() ? b : t); };
    var orElseGetTrue = function () { return orElseGet(true); };
    var orElseGetFalse = function () { return orElseGet(false); };
    return __assign(__assign({}, (0, parser_1.createPrimitiveParser)({
        t: t,
        isMatchingType: isMatchingType,
        expectedType: 'boolean'
    })), { orElseGetTrue: orElseGetTrue, orElseGetFalse: orElseGetFalse, exactlyAs: function (b) { return (__assign(__assign({}, (0, option_1.createExact)({
            t: t,
            e: b,
            expectedType: 'boolean'
        })), { orElseGetTrue: orElseGetTrue, orElseGetFalse: orElseGetFalse })); } });
};
exports.parseAsBoolean = parseAsBoolean;
var parseAsNumber = function (t) {
    var expectedType = 'number';
    var isMatchingType = function () { return (0, util_1.isEqual)(expectedType, typeof t); };
    return __assign(__assign({}, (0, parser_1.createPrimitiveParser)({
        t: t,
        expectedType: expectedType,
        isMatchingType: isMatchingType
    })), { inRangeOf: function (min, max) {
            if (min > max || max < min) {
                throw new Error("min must be less than max, max must also be greater than min. Min: ".concat(min, ", Max: ").concat(max));
            }
            var options = (0, option_1.createOptions)({
                t: function () { return t; },
                expectedType: expectedType,
                isMatchingType: function () {
                    if (!isMatchingType()) {
                        return false;
                    }
                    var n = t;
                    return n >= min && n <= max;
                }
            });
            return options;
        }, exactlyAs: function (n) { return (__assign({}, (0, option_1.createExact)({
            t: t,
            e: n,
            expectedType: expectedType
        }))); } });
};
exports.parseAsNumber = parseAsNumber;
/**
 * Although it can be used to parse anything, this is recommended to be used to parse as a specified non-object type
 * @returns the Options object
 */
var parseAsCustomType = function (t, isParseable) {
    return (0, option_1.createOptions)({
        t: function () { return t; },
        isMatchingType: function () { return isParseable(t); },
        expectedType: (0, util_1.getPossibleSelfDefinedType)(t)
    });
};
exports.parseAsCustomType = parseAsCustomType;
//# sourceMappingURL=primitive.js.map